/* use x86 cycle counter for fast, high-resolution timing */
/* Mark Moraes */
#include "hrtimer_x86.h"
#include <stdio.h>
#include <sys/time.h>

#define CALIBRATE_USECS 10000
#define USECS_PER_SEC (1000L*1000L)

typedef unsigned long ulong;

static int debug;

uint64_t hrtimer_x86(void)
{
    uint64_t val;
    HRTIMER_X86_GET(val);
    return val;
}

uint64_t hrtimer_x86_calibrate(void)
{
    uint64_t t1;
    uint64_t t2;
    struct timeval tv1, tv2, tvr;
    uint64_t k = 0;

    t1 = hrtimer_x86();
    gettimeofday(&tv1, 0);
    if (debug)
	printf("%s: %lu %lu %llu\n",  __FUNCTION__,
	       (ulong) tv1.tv_sec, (ulong) tv1.tv_usec,
	       (unsigned long long) t1);
    do {
	t2 = hrtimer_x86();
	gettimeofday(&tv2, 0);
	timersub(&tv2, &tv1, &tvr);
	if (debug)
	    printf("%s: %lu %lu %llu : %lu %lu %llu\n", __FUNCTION__,
		   (ulong) tv2.tv_sec, (ulong) tv2.tv_usec,
		   (unsigned long long) t2,
		   (ulong) tvr.tv_sec, (ulong) tvr.tv_usec,
		   (unsigned long long) (t2 - t1));
    } while (tvr.tv_sec == 0 && tvr.tv_usec < CALIBRATE_USECS);
    return (t2 - t1) / ((uint64_t) tvr.tv_usec +
			((uint64_t) tvr.tv_sec)*USECS_PER_SEC);
}

int hrtimer_debug(int debugval)
{
    int olddebug = debug;
    debug = debugval;
    return olddebug;
}
