#include <stdio.h>
#include <stdarg.h>
#include <errno.h>
#include <limits.h>
#include <mpi.h>
#include "mpi_err.h"

#ifndef HOST_NAME_MAX
# ifdef MAXHOSTNAMELEN
#  define HOST_NAME_MAX MAXHOSTNAMELEN
# else
#  define HOST_NAME_MAX 64
# endif
#endif

static char rankstr[HOST_NAME_MAX+sizeof(int)*3+2];

void
errinit()
{
  int e, rank = -1;
  char hname[HOST_NAME_MAX];
  (void) gethostname(hname, sizeof(hname));
  e = MPI_Comm_rank( MPI_COMM_WORLD, &rank );
  snprintf(rankstr, sizeof(rankstr), "%d:%s: ", rank, hname);
  if (e != MPI_SUCCESS)
    error(100, "MPI_Comm_Rank failed: %d", e);
}

void
vout(const char *fmt, va_list ap)
{
  if (fmt) {
    fputs(rankstr, stderr);
    vfprintf(stderr, fmt, ap);
    putc('\n', stderr);
  }
}

void
verror(int exitstatus, const char *fmt, va_list ap)
{
  int e = errno;
  vout(fmt, ap);
  if (e != 0)
    fprintf(stderr, "%s: %s\n", rankstr, strerror(e));
  MPI_Abort(MPI_COMM_WORLD, exitstatus);
  exit(exitstatus); /* may not be reached */
  /* NOTREACHED */
}

void
error(int exitstatus, const char *fmt, ...)
{
  va_list ap;
  va_start(ap, fmt);
  verror(exitstatus, fmt, ap);
  va_end(ap); /* not reached! */
}
