#!/bin/sh
# Run MPI latency/bandwidth test over range of parameters:
# Packet size, type of test (overlap, barrier, reduce, no overlap), neighbours
# Mark Moraes, D. E. Shaw Research & Development, LLC
#
##### The following two lines may need to be changed for the MPI version #####
# Or they can just be overridden by setting the environment variables
# MAXNODES or MPIRUNCMD appropriately.
# : ${MAXNODES:=`lamnodes | wc -l`}	# total processors available
HOSTFILE=${1-$HOME/desrad-drda-nj-ib}	# list of nodes
: ${MAXNODES:=`wc -l < $HOSTFILE`}	# total processors available
: ${MPIRUNCMD:="python ../qdrun/mvapich.py -p param.topspin-20050826 -E 30 -T 3600"}		# run an MPI job on specified number of
					# processors, which will be appended to
					# this string. e.g. "mpiexec -n"
#####
maxiter=100000		# default is for high bandwidth, low latency
				# (Altix, Quadrics, Infiniband, Myrinet)
nvals="1 100 500 1000 5000 10000"
case "$MAXNODES" in 
0|'')	echo "Error: MAXNODES returned \"$MAXNODES\"" >&2; exit 1;;
esac
if test `expr $MAXNODES % 2` -ne 0; then
	MAXNODES=`expr $MAXNODES - 1`
fi
k=2
sizelist=
sep=
nl='
'
while test $k -le $MAXNODES; do
	sizelist="${sizelist}${sep}${k}"
	sep="$nl"
	nk=`expr $k \* 2`
	nk=`expr $k \* 2`
	if test $k -lt $MAXNODES -a $nk -gt $MAXNODES; then
		k=$MAXNODES
	else
		k=$nk
	fi
done
sizelist=`echo "$sizelist" | sort -rn`
# echo $sizelist
oprefix="`date +%Y%m%d_%H%M%S`_$$"
ofile="${oprefix}.log"
odir="${oprefix}_stats"
mkdir "$odir"
echo "Saving output to $ofile"
(
echo "+++ OS:"
uname -a
echo "+++ CPU Info:"
cat /proc/cpuinfo
echo "+++ Memory Info:"
cat /proc/meminfo
#increasing maxneighbours is a good way to stress the fabric
#maxneighbours=`expr $MAXNODES / 2`
maxneighbours=2
for ov in 1 2 3 0; do
	kn=1
	while test $kn -le $maxneighbours; do
		for k in $sizelist; do
			if test `expr $kn \* 2` -gt $k; then
				continue
			fi
			for i in $nvals; do
				# scale iter down a bit as we increase the size of the exchange
				# to-do: we would scale the multiple of floats*iterations*neighbours to be roughly constant, factor in delay for overlap
				#iter=`awk 'BEGIN {i = '$i'; maxiter='$maxiter'; if (i > 1) print int(maxiter / log(i) / 2.303 + 1); else print maxiter; }'`
				iter=$maxiter
				echo ">>> $k processors: floats=$i iterations=$iter overlap=$ov neighbours=$kn"
				$MPIRUNCMD -n $k $HOSTFILE ./mpi_latency_ib_moraes $i $iter $ov $kn $odir
			done
		done
		nk=`expr $kn \* 2`
		if test $kn -lt $maxneighbours -a $nk -gt $maxneighbours; then
			kn=$maxneighbours
		else
			kn=$nk
		fi
	done
done
) 2>&1 | tee "$ofile"
